package com.qmth.themis.business.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TEExamSummaryMapper;
import com.qmth.themis.business.entity.TEExamSummary;
import com.qmth.themis.business.service.TEExamActivityService;
import com.qmth.themis.business.service.TEExamSummaryService;
import com.qmth.themis.business.service.ThemisCacheService;
import com.qmth.themis.business.util.UidUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;
import java.util.Set;

/**
 * <p>
 * 考试信息统计表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-10-27
 */
@Service
public class TEExamSummaryServiceImpl extends ServiceImpl<TEExamSummaryMapper, TEExamSummary> implements TEExamSummaryService {
    private final static Logger log = LoggerFactory.getLogger(TEExamSummaryServiceImpl.class);

    @Resource
    TEExamActivityService teExamActivityService;

    @Resource
    ThemisCacheService themisCacheService;

    /**
     * 统计考试信息
     *
     * @param examId
     * @param examActivityId
     * @param roomCodeSet
     */
    @Override
    public void examSummary(Long examId, Long examActivityId, Set<String> roomCodeSet) {
        if (!CollectionUtils.isEmpty(roomCodeSet)) {
            for (String s : roomCodeSet) {
                //根据考试批次、场次、考场编码统计信息
                TEExamSummary teExamSummary = this.baseMapper.examSummary(examId, examActivityId, s);
                this.examSummaryCommon(examId, examActivityId, s, teExamSummary);
            }
        }
    }

    /**
     * 考试计算公用
     *
     * @param examId
     * @param examActivityId
     * @param s
     * @param teExamSummary
     */
    protected void examSummaryCommon(Long examId, Long examActivityId, String s, TEExamSummary teExamSummary) {
        if (Objects.nonNull(teExamSummary) && teExamSummary.getTotalCount().intValue() > 0) {
            teExamSummary.setExamId(examId);
            teExamSummary.setExamActivityId(examActivityId);
            teExamSummary.setRoomCode(s);
            ExamActivityCacheBean ac = teExamActivityService.getExamActivityCacheBean(examActivityId);
            //换算开始时间、侯考时间、结束时间
            Long startTime = ac.getStartTime() - (ac.getPrepareSeconds() * 1000);
            Integer openingSecondsTemp = ac.getOpeningSeconds();
            openingSecondsTemp = Objects.nonNull(openingSecondsTemp) && openingSecondsTemp.intValue() == 0 ? SystemConstant.DEFAULT_OPENING_SECONDS : openingSecondsTemp;
            Long finalPrepareTime = ac.getStartTime() + (openingSecondsTemp * 1000);
            Long endTime = ac.getFinishTime() + SystemConstant.FINISH_DELAY_TIME;//交卷时间加5分钟，后台交卷可能未完成
            long timestamp = System.currentTimeMillis();
            if (startTime <= timestamp || teExamSummary.getPrepareCount().intValue() > 0) {//侯考
                teExamSummary.setAbsentCount(teExamSummary.getTotalCount() - teExamSummary.getPrepareCount());
            }
            log.info("examSummary examId:{},examActivityId:{},startTime:{},finalPrepareTime:{},endTime:{}", examId, ac.getId(), DateUtil.format(new Date(startTime), "yyyy-MM-dd HH:mm:ss"), DateUtil.format(new Date(finalPrepareTime), "yyyy-MM-dd HH:mm:ss"), DateUtil.format(new Date(endTime), "yyyy-MM-dd HH:mm:ss"));
            //最后侯考时间已过且未结束考试，当前侯考数设为0
            if (finalPrepareTime <= timestamp) {
                teExamSummary.setPrepareCount(0);
            }
            //交卷
            if (endTime <= timestamp) {//当考试场次结束时间已过，缺考=全部应考-已完成考试
                teExamSummary.setAbsentCount(teExamSummary.getTotalCount() - teExamSummary.getFinishCount());
            } else {//否则缺考=全部应考-侯考数(0)-考试中-已完成考试
                teExamSummary.setAbsentCount(teExamSummary.getTotalCount() - teExamSummary.getPrepareCount() - teExamSummary.getExamCount() - teExamSummary.getFinishCount());
            }
            TEExamSummary teExamSummaryDb = this.getOne(new QueryWrapper<TEExamSummary>().lambda()
                    .eq(TEExamSummary::getExamId, examId)
                    .eq(TEExamSummary::getExamActivityId, examActivityId)
                    .eq(TEExamSummary::getRoomCode, s));
            if (Objects.isNull(teExamSummaryDb)) {
                teExamSummaryDb = new TEExamSummary();
                BeanUtils.copyProperties(teExamSummary, teExamSummaryDb);
                teExamSummaryDb.setId(UidUtil.nextId());
            } else {
                Long id = teExamSummaryDb.getId();
                BeanUtils.copyProperties(teExamSummary, teExamSummaryDb);
                teExamSummaryDb.setId(id);
            }
            this.saveOrUpdate(teExamSummaryDb);
            themisCacheService.updateExamSummaryCache(examId);
            themisCacheService.updateExamSummaryCache(examId, examActivityId);
            themisCacheService.updateExamSummaryCache(examId, examActivityId, s);
        }
    }
}
